@@ -16,15 +16,19 @@ from __future__ import absolute_import, print_function, unicode_literals
 import argparse
 import codecs
 import locale
+import logging
 from pkg_resources import iter_entry_points
 import sys
 import traceback
 
-from reprounzip.common import setup_logging
+from reprounzip.common import setup_logging, \
+    setup_usage_report, enable_usage_report, \
+    submit_usage_report, record_usage
 from reprounzip import signals
+from reprounzip.unpackers.common import UsageError
 
 
-__version__ = '0.5'
+__version__ = '0.5.1'
 
 
 unpackers = {}
@@ -50,6 +54,21 @@ def get_plugins(entry_point_name):
         yield name, func, descr, descr_1
 
 
+class RPUZArgumentParser(argparse.ArgumentParser):
+    def error(self, message):
+        sys.stderr.write('error: %s\n' % message)
+        self.print_help(sys.stderr)
+        sys.exit(2)
+
+
+def usage_report(args):
+    if bool(args.enable) == bool(args.disable):
+        logging.critical("What do you want to do?")
+        raise UsageError
+    enable_usage_report(args.enable)
+    sys.exit(0)
+
+
 def main():
     """Entry point when called on the command-line.
     """
@@ -67,15 +86,6 @@ def main():
     else:
         sys.stdin = sys.stdin.buffer
 
-    # http://bugs.python.org/issue13676
-    # This prevents reprozip from reading argv and envp arrays from trace
-    if sys.version_info < (2, 7, 3):
-        sys.stderr.write("Error: your version of Python, %s, is not "
-                         "supported\nVersions before 2.7.3 are affected by "
-                         "bug 13676 and will not work with ReproZip\n" %
-                         sys.version.split(' ', 1)[0])
-        sys.exit(1)
-
     # Parses command-line
 
     # General options
@@ -90,7 +100,7 @@ def main():
     for name, func, descr, descr_1 in get_plugins('reprounzip.plugins'):
         func()
 
-    parser = argparse.ArgumentParser(
+    parser = RPUZArgumentParser(
             description="reprounzip is the ReproZip component responsible for "
                         "unpacking and reproducing an experiment previously "
                         "packed with reprozip",
@@ -98,6 +108,14 @@ def main():
             parents=[options])
     subparsers = parser.add_subparsers(title="subcommands", metavar='')
 
+    # usage_report subcommand
+    parser_stats = subparsers.add_parser(
+            'usage_report', parents=[options],
+            help="Enables or disables anonymous usage reports")
+    parser_stats.add_argument('--enable', action='store_true')
+    parser_stats.add_argument('--disable', action='store_true')
+    parser_stats.set_defaults(func=usage_report)
+
     # Loads unpackers
     for name, func, descr, descr_1 in get_plugins('reprounzip.unpackers'):
         plugin_parser = subparsers.add_parser(
@@ -118,13 +136,28 @@ def main():
     except AttributeError:
         signals.unpacker = None
     setup_logging('REPROUNZIP', args.verbosity)
+
+    setup_usage_report('reprounzip', __version__)
+    if hasattr(args, 'selected_unpacker'):
+        record_usage(unpacker=args.selected_unpacker)
+    signals.pre_setup.subscribe(lambda **kw: record_usage(setup=True))
+    signals.pre_run.subscribe(lambda **kw: record_usage(run=True))
+
     try:
-        args.func(args)
-    except Exception as e:
-        signals.application_finishing(reason=e)
-        raise
-    else:
-        signals.application_finishing(reason=None)
+        try:
+            args.func(args)
+        except UsageError:
+            raise
+        except Exception as e:
+            signals.application_finishing(reason=e)
+            submit_usage_report(result=type(e).__name__)
+            raise
+        else:
+            signals.application_finishing(reason=None)
+    except UsageError:
+        parser.print_help(sys.stderr)
+        sys.exit(1)
+    submit_usage_report(result='success')
     sys.exit(0)
 
 
