@@ -26,13 +26,13 @@ import subprocess
 import sys
 import tarfile
 
-from reprounzip.common import load_config as load_config_file
+from reprounzip.common import load_config as load_config_file, record_usage
 from reprounzip import signals
 from reprounzip.unpackers.common import THIS_DISTRIBUTION, PKG_NOT_INSTALLED, \
     COMPAT_OK, COMPAT_NO, target_must_exist, shell_escape, load_config, \
     select_installer, busybox_url, join_root, FileUploader, FileDownloader, \
     get_runs
-from reprounzip.utils import unicode_, iteritems, itervalues, \
+from reprounzip.utils import unicode_, irange, iteritems, itervalues, \
     make_dir_writable, rmtree_fixed, download_file
 
 
@@ -68,6 +68,7 @@ def installpkgs(args):
             packages = [pkg for pkg in packages if not pkg.packfiles]
 
         # Installs packages
+        record_usage(installpkgs_installing=len(packages))
         r, pkgs = installer.install(packages, assume_yes=args.assume_yes)
         for pkg in packages:
             req = pkg.version
@@ -116,7 +117,7 @@ def directory_create(args):
         logging.critical("Target directory exists")
         sys.exit(1)
 
-    if DefaultAbstractPath is not PosixPath:
+    if not issubclass(DefaultAbstractPath, PosixPath):
         logging.critical("Not unpacking on POSIX system")
         sys.exit(1)
 
@@ -149,6 +150,7 @@ def directory_create(args):
                         f, pkg.name)
                 missing_files = True
     if missing_files:
+        record_usage(directory_missing_pkgs=True)
         logging.error(
                 "Some packages are missing, you should probably install "
                 "them.\nUse 'reprounzip installpkgs -h' for help")
@@ -213,7 +215,7 @@ def directory_run(args):
 
     selected_runs = get_runs(runs, args.run, cmdline)
 
-    root = target / 'root'
+    root = (target / 'root').absolute()
 
     # Gets library paths
     lib_dirs = []
@@ -258,6 +260,23 @@ def directory_run(args):
         # FIXME : Use exec -a or something if binary != argv[0]
         if cmdline is None:
             argv = run['argv']
+
+            # Rewrites command-line arguments that are absolute filenames
+            rewritten = False
+            for i in irange(len(argv)):
+                try:
+                    p = Path(argv[i])
+                except UnicodeEncodeError:
+                    continue
+                if p.is_absolute:
+                    rp = join_root(root, p)
+                    if (rp.exists() or
+                            (len(rp.components) > 3 and rp.parent.exists())):
+                        argv[i] = str(rp)
+                        rewritten = True
+            if rewritten:
+                logging.warning("Rewrote command-line as: %s",
+                                ' '.join(shell_escape(a) for a in argv))
         else:
             argv = cmdline
         cmd += ' '.join(shell_escape(a) for a in argv)
@@ -296,20 +315,23 @@ def should_restore_owner(param):
             # Nothing was requested
             logging.warning("Not running as root, won't restore files' "
                             "owner/group")
+            ret = False
         else:
             # If False: skip warning
-            return False
+            ret = False
     else:
         if param is None:
             # Nothing was requested
             logging.info("Running as root, we will restore files' "
                          "owner/group")
-            return True
+            ret = True
         elif param is True:
-            return True
+            ret = True
         else:
             # If False: skip warning
-            return False
+            ret = False
+    record_usage(restore_owner=ret)
+    return ret
 
 
 def should_mount_magic_dirs(param):
@@ -324,19 +346,22 @@ def should_mount_magic_dirs(param):
         elif param is None:
             # Nothing was requested
             logging.warning("Not running as root, won't mount /dev and /proc")
+            ret = False
         else:
             # If False: skip warning
-            return False
+            ret = False
     else:
         if param is None:
             # Nothing was requested
             logging.info("Running as root, will mount /dev and /proc")
-            return True
+            ret = True
         elif param is True:
-            return True
+            ret = True
         else:
             # If False: skip warning
-            return False
+            ret = False
+    record_usage(mount_magic_dirs=ret)
+    return ret
 
 
 def chroot_create(args):
@@ -383,6 +408,7 @@ def chroot_create(args):
     # Checks that everything was packed
     packages_not_packed = [pkg for pkg in packages if not pkg.packfiles]
     if packages_not_packed:
+        record_usage(chroot_missing_pkgs=True)
         logging.warning("According to configuration, some files were left out "
                         "because they belong to the following packages:%s"
                         "\nWill copy files from HOST SYSTEM",
@@ -468,7 +494,7 @@ def chroot_mount(args):
         d = join_root(target / 'root', Path(m))
         d.mkdir(parents=True)
         logging.info("Mounting %s on %s...", m, d)
-        subprocess.check_call(['mount', '--bind', m, str(d)])
+        subprocess.check_call(['mount', '-o', 'rbind', m, str(d)])
 
     write_dict(target / '.reprounzip', {'mounted': True}, 'chroot')
 
@@ -521,17 +547,14 @@ def chroot_run(args):
     signals.post_run(target=target, retcode=retcode)
 
 
-@target_must_exist
-def chroot_destroy_unmount(args):
-    """Unmounts the bound magic dirs.
+def chroot_unmount(target):
+    """Unmount magic directories, if they are mounted.
     """
-    target = Path(args.target[0])
     unpacked_info = read_dict(target / '.reprounzip', 'chroot')
     mounted = unpacked_info.get('mounted', False)
 
     if not mounted:
-        logging.critical("Magic directories were not mounted")
-        sys.exit(1)
+        return False
 
     unpacked_info['mounted'] = False
     write_dict(target / '.reprounzip', unpacked_info, 'chroot')
@@ -540,7 +563,25 @@ def chroot_destroy_unmount(args):
         d = join_root(target / 'root', Path(m))
         if d.exists():
             logging.info("Unmounting %s...", d)
-            subprocess.check_call(['umount', str(d)])
+            # Unmounts recursively
+            subprocess.check_call(
+                    'grep %s /proc/mounts | '
+                    'cut -f2 -d" " | '
+                    'sort -r | '
+                    'xargs umount' % d,
+                    shell=True)
+    return True
+
+
+@target_must_exist
+def chroot_destroy_unmount(args):
+    """Unmounts the bound magic dirs.
+    """
+    target = Path(args.target[0])
+
+    if not chroot_unmount(target):
+        logging.critical("Magic directories were not mounted")
+        sys.exit(1)
 
 
 @target_must_exist
@@ -565,14 +606,8 @@ def chroot_destroy(args):
     """Destroys the directory, unmounting first if necessary.
     """
     target = Path(args.target[0])
-    mounted = read_dict(target / '.reprounzip', 'chroot').get('mounted', False)
 
-    if mounted:
-        for m in ('/dev', '/proc'):
-            d = join_root(target / 'root', Path(m))
-            if d.exists():
-                logging.info("Unmounting %s...", d)
-                subprocess.check_call(['umount', str(d)])
+    chroot_unmount(target)
 
     logging.info("Removing directory %s...", target)
     signals.pre_destroy(target=target)
@@ -639,6 +674,10 @@ class LocalDownloader(FileDownloader):
         remote_path = join_root(self.root, remote_path)
 
         # Output to stdout
+        if not remote_path.exists():
+            logging.critical("Can't get output file (doesn't exist): %s",
+                             remote_path)
+            sys.exit(1)
         with remote_path.open('rb') as fp:
             chunk = fp.read(1024)
             if chunk:
@@ -652,6 +691,10 @@ class LocalDownloader(FileDownloader):
         remote_path = join_root(self.root, remote_path)
 
         # Copy
+        if not remote_path.exists():
+            logging.critical("Can't get output file (doesn't exist): %s",
+                             remote_path)
+            sys.exit(1)
         remote_path.copyfile(local_path)
         remote_path.copymode(local_path)
 
